//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Material/MaterialItemUtils.cpp
//! @brief     Implements class MaterialItemUtils
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/Material/MaterialItemUtils.h"
#include "GUI/Model/Material/MaterialItem.h"
#include "GUI/Model/Material/MaterialItemContainer.h"
#include "GUI/Model/Material/MaterialModel.h"
#include "GUI/Model/Material/MaterialModelStore.h"
#include "GUI/Util/Error.h"
#include "Sample/Material/Material.h"

#include <deque>
#include <random>

QColor GUI::MaterialUtil::randomMaterialColor()
{
    static std::random_device r;
    std::default_random_engine re(r());
    std::uniform_int_distribution<int> ru(0, 255);

    return QColor(ru(re), ru(re), ru(re));
}

QString GUI::MaterialUtil::defaultMaterialIdentifier()
{
    if (!GUI::Model::MaterialModelStore::materialModel())
        return QString();

    auto materials = GUI::Model::MaterialModelStore::materialModel()->topItems<MaterialItem>();
    return materials.isEmpty() ? QString() : materials.front()->identifier();
}

std::unique_ptr<Material> GUI::MaterialUtil::createDomainMaterial(const QString& materialIdentifier)
{
    MaterialItem* materialItem = findMaterial(materialIdentifier);
    return materialItem->createMaterial();
}

std::unique_ptr<Material>
GUI::MaterialUtil::createDomainMaterial(const QString& materialIdentifier,
                                        const MaterialItemContainer& container)
{
    const MaterialItem* material_item = container.findMaterialById(materialIdentifier);
    if (!material_item)
        throw Error("MaterialUtils::createDomainMaterial() -> Error. Can't find "
                    "material with identifier '"
                    + materialIdentifier + "'.");
    return material_item->createMaterial();
}

MaterialItem* GUI::MaterialUtil::findMaterial(const QString& materialIdentifier)
{
    if (!GUI::Model::MaterialModelStore::materialModel())
        throw Error("GUI::MaterialUtil::findMaterial() -> Error. "
                    "Attempt to access non-existing material model");

    auto* material =
        GUI::Model::MaterialModelStore::materialModel()->materialFromIdentifier(materialIdentifier);

    if (!material)
        throw Error("MaterialUtils::findMaterial() -> Error. Can't find "
                    "material with identifier '"
                    + materialIdentifier + "'.");
    return material;
}
